{{  David Ratcliff   'ratronic'   5-3-10  }}
con
   _clkmode = xtal1 + pll16x            'system clock 
   _xinfreq = 5_000_000
   #1, i_Ping, i_ReadData, i_WriteData, i_RegWrite, i_Action, i_Reset, #131, i_SyncWrite
   adc = 0                              'adc channel
   dyn = 8                              'i/o pin for dynamixel servo bus
   grf = 1500                           'gripper force limit 
   gro = 560                            'gripper open limit
   grc = 1000                           'gripper close limit
   
var
  byte byteArray[50],gf,lf,wf
  word ar,bt,bs,fa,gr,ws
   
obj
    
  dy : "dynacom"
  tv : "vga_text"
  ms : "mouse"
  ad : "cd_ltc1298"
  
pub arm | i

  'initialization
  
  ad.start(0)                           'start adc                            
  tv.start(16)                          'start vga display
  tv.str(string(1,"Wait 3 seconds"))    'display 'wait' for servos to center
  ms.start(24,25)                       'start mouse
  dy.start(8)                           'start dynamixel bus on propeller i/o pin#
  ms.bound_scales(1,1,1)                'set mouse bounding scales  
  ms.bound_limits(0,0,0,1023,1023,1023) 'set mouse bound min max limits for x, y, and scrollwheel(z)  
  ms.bound_preset(512,512,512)          'set mouse preset start values
  ws := 512                             'wrist                                                               
  gr := gro                             'gripper
  repeat i from 1 to 5                  'set moving speed on servos 1-5 (100 is just write for me)
    dy.setsrvspd(i, 200)  
  repeat i from 1 to 4                  'set servos 1-4 w/ 512 - center all servo positions except gripper     
    dy.setsrvpos(i, 512)                '                                                    
  dy.writedata(5,18,1)                  'disable overload shut down for gripper
                         
  'main loop
  
  repeat                                
    bt := ms.buttons                    'get mouse button 
    bs := ms.bound_x                    'get mouse x (base position)
    ar := ms.bound_y                    'get mouse y (arm position)
    fa := 512 + ms.abs_z * 5            'get mouse scroll wheel (forearm position)
    if lf == 0                          'level flag
      fa := fa - ms.abs_y               'keep forearm level with upper arm movement   
    if bt == 1                          'if mouse left button - open / close gripper 
      gripper
      !gf                               'toggle gripper direction flag
    if bt == 2                          'if mouse right button - move wrist cw / ccw 
      wrist
      !wf
    if bt == 4                          'if scroll wheel button pressed toggle level flag
      level
      
  'set min/max limits
            
    ar := 120 #> ar <# 620              'arm 
    bs := 390 #> bs <# 1023             'base        
    fa := 120 #> fa <# 910              'forearm
    gr := gro #> gr <# grc              'gripper
    ws := 0   #> ws <# 1023             'wrist
       
  'write position values to servos
    dy.setsrvpos(1, bs)                 'set base servo position
    dy.setsrvpos(2, ar)                 'set arm servo position
    dy.setsrvpos(3, fa)                 'set forearm servo position
    dy.setsrvpos(4, ws)                 'set wrist servo position
    dy.setsrvpos(5, gr)                 'set gripper servo position
    
    display                             'display all values
     
pub display

  'update screen with all values
  
  tv.out(1)
  tv.str(string("Servo positions",13))                         
  tv.str(string("Base      = "))    
  tv.dec(bs)                        
  tv.str(string("     ",13))        
  tv.str(string("Arm       = "))    
  tv.dec(ar)                        
  tv.str(string("     ",13))        
  tv.str(string("Forearm   = "))    
  tv.dec(fa)                        
  tv.str(string("     ",13))        
  tv.str(string("Wrist     = "))    
  tv.dec(ws)                        
  tv.str(string("     ",13))        
  tv.str(string("Gripper   = "))    
  tv.dec(gr)                        
  tv.str(string("     ",13,13))        
  tv.str(string("Gripper load = " ))   
  tv.dec(4095 - ad.getadc(adc))              
  tv.str(string("     ",13))        

pub gripper

  'toggle gripper open or closed
  
  repeat
    if gf == 0                          'close gripper
      if gr == grc                      'if gripper closed then stop
        return
      if 4095 - ad.getadc(adc) > grf    'if force sensor at limit then stop
        return                            
      gr++                              'increase gripper position   
      if gr == grc                      'if gripper closed then stop
        return                            
      
      dy.setsrvpos(5, gr)               'set servo id#5 to gr position
           
    if gf == 255                        'open gripper
      if gr == gro                      'if gripper open then stop
        return
      gr := gro                         'move gripper servo
      
      dy.setsrvpos(5, gr)
      waitcnt(80000000 + cnt)
      return                              
  
pub level

  'toggle level flag
  
  if lf == 0
    !lf
    return
  if lf == 255
    !lf
    return  
          
pub wrist

  'toggle wrist clockwise/counterclockwise
  
  repeat
    if wf == 0                          'rotate wrist counterclockwise
      if ws == 1023                     'if wrist rotated then stop
        return
      if ms.buttons == 0                'if mouse button not pressed then stop
        return                  
      word[@bytearray[0]] := ws         'move wrist servo
      dy.setsrvpos(4, ws)          
      ws++                        
      if ws == 1023                     'if wrist at end then stop
        return
      waitcnt(500000 + cnt)  
    if wf == 255                        'rotate wrist clockwise
      if ws == 0                        'if wrist rotated then stop
        return
      if ms.buttons == 0                'if mouse button not pressed then stop
        return
      word[@bytearray[0]] := ws         'move wrist servo
      dy.setsrvpos(4, ws)          
      ws--                        
      if ws == 0                        'if wrist at end then stop
        return
      waitcnt(500000 + cnt)                   
                                                                    